<div class="mx-auto md:w-2/3 w-full">
  <h1 class="font-bold text-4xl">Editing person</h1>


    <div class="flex flex-row justify-between items-center py-2">
        <div class="flex flex-row items-center">
            <div class="flex flex-col">
                <div class="text-lg font-medium text-gray-900 dark:text-gray-100">
                    <a
                    href="/persons/{{ person.id }}"
                    class="rounded-lg py-3 px-5 bg-gray-100 inline-block font-medium"
                    >Show this person</a
                    >
                </div>
            </div>
        </div>
    </div>

    <div class="flex flex-row justify-between items-center py-2">
        <div class="flex flex-row items-center">
            <div class="flex flex-col">
                <div class="text-lg font-medium text-gray-900 dark:text-gray-100">
                    <a
                    href="/persons"
                    class="rounded-lg py-3 px-5 bg-gray-100 inline-block font-medium"
                    >Back to persons</a
                    >
                </div>
            </div>
        </div>
    </div>
</div>
