{% extends "base.html" %} {% block content %}
<main class="mx-auto max-w-4xl">
  <section
    class="text-black dark:text-white widescreen:section-min-height tallscreen:section-min-height justify-center items-center"
  >
    <div class="text-5xl p-5">Page not found</div>
    <p class="p-5">
      Sorry, but the page you were trying to view does not exist.
    </p>
    <div class="p-5 text-primary"><a href="/">Go Home</a></div>
  </section>
</main>
{% endblock %}
