<head>
  <meta charset="UTF-8" />
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  <meta name="description" content="Snok is a full-stack python framework designed for productivity and fun." />
  <title>Snok</title>
  <link
    rel="shortcut icon"
    href="/static/images/favicon.ico"
    type="image/x-icon"
  />
  <link href="/static/css/tailwind.css" rel="stylesheet" />
  <script src="/static/js/htmx/1.9.2/htmx.min.js"></script>
  <script src="/static/js/htmx/1.9.2/htmx-json-enc.min.js"></script>
</head>
