<footer id="footer" class="text-sm text-white">
  <section
    class="mx-auto flex max-w-4xl flex-col p-4 sm:flex-row sm:justify-between"
  >
    <nav class="flex gap-5" aria-label="footer">
      <a
        href="https://github.com/anthonycorletti/snok"
        target="_blank"
        class="hover:opacity-90 text-lg text-black dark:text-white"
        >GitHub 🐍</a
      >
      <a
        href="https://snok.corletti.xyz"
        target="_blank"
        class="hover:opacity-90 text-lg text-black dark:text-white"
        >Docs 🐍</a
      >
    </nav>
  </section>
</footer>
